
;===============

TestFor32BitCPU proc	far

;-------------------------------------------------------------------------------
;Should identify all x86 processors, using the process of elimination.
;Detects if a NPX/FPU is present.
;-------------------------------------------------------------------------------

PUSHCONTEXT CPU

.586P
		call	TestCPU
		call	TestNPX

		cmp	[CPU], 3
		jae	CPU_32Bit

		call	Emit_CPU_NPX
		error	<Requires 32-bit CPU.>

DOSExit_ErrorCode:

                @DOSExit 1
CPU_32Bit:
                ret

TestFor32BitCPU endp

TestCPU 	proc

		call	TestCPU_AL
		mov	[CPU], al
TestCPU_AL:

;Test for 8086.
                push    sp
                pop     ax
                cmp     sp,ax
		je	Not_8086
		mov	al, 0			;8086
                ret
Not_8086:

;Test for 80286.
                pushf
                pop     ax
                or      ax, 4000h
                push    ax
                popf
                pushf
                pop     ax
                test    ax, 4000h
		jnz	Not_286
		mov	al, 2			;286
                ret
Not_286:

;Test for 386.
;Check if AC bit in EFLAGS can be changed (if so it's not a 386).

		mov	[tempd], esp		;save esp
		and	esp, not 3		;align to dword

                pushfd
                pop     eax

                mov     ebx, eax

                xor     eax, 40000h

                push    eax
                popfd

                pushfd
                pop     eax

                xor     eax, ebx
		jnz	Not_386

		mov	al, 3			;386
		jmp	CPU_Is_32Bit
Not_386:

;Test if CPU supports the CPUID instruction. If not, then CPU is a 486.
;Check if ID flag (Bit 21) in EFLAGS can be toggled.
;If so, then the CPUID instruction is supported, which will output the CPU
;family number in bits 11..8 of EAX when EAX is loaded with 1 before CPUID.

		push	ebx			;restore 486's AC bit from 386 test
		popfd				;(386 doesn't have a AC bit)

		mov	eax, ebx		;get saved EFLAGS from 386 test
		xor	eax, 1 shl 21		;flip ID bit in EFLAGS (bit 21)
                push    eax
                popfd
                pushfd
                pop     eax
		xor	eax, ebx		;XOR tests if toggled
		jz	CPU_Is_486_CPUIDless	;=486 if ID bit stayed the same ZF=1

		mov	[CPUID_Implemented], -1 ;CPUID is implemented

		mov	eax, 1
		cpuid

		and	eax, 111100000000b	;family=bits 11..8
		shr	eax, 8

		jmp	CPU_Is_32Bit

CPU_Is_486_CPUIDless:

		mov	al, 4			;486

                ;-- fall thru --
CPU_Is_32Bit:

;For 32-bit CPU's, restore saved ESP from the 386 test.
;Exit here for 32-bit CPU tests.

		mov	esp, [tempd]

                ret

TestCPU 	endp

;===============

TestNPX 	proc

;-------------------------------------------------------------------------------
;Sets [NPX_Flag] if am NPX/FPU is present.
;-------------------------------------------------------------------------------

		fninit
		mov	[tempw], 5A5Ah
		fnstsw	[tempw]

		.IF	[tempw] == 0
		mov	[NPX_Flag], -1
		.ENDIF

		ret

TestNPX 	endp

Emit_CPU_NPX	proc

		mov	al, [CPU]
		mov	ah, [NPX_Flag]
_Emit = 1

	.IF	al == 0
		emit	<CPU = 8086>
		.IF	ah
		emit	<NPX = 8087>
		.ENDIF
	.ENDIF

	.IF	al == 2
		emit	<CPU = 80286>
		.IF	ah
		emit	<NPX = 80287>
		.ENDIF
	.ENDIF

	.IF	al == 3
		emit	<CPU = 80386>
		.IF	ah
		emit	<NPX = 80387/80287>
		.ENDIF
	.ENDIF

	.IF	al == 4
		.IF   ah
		emit   <CPU = 80486DX>
		.ELSE
		emit   <CPU = 80486SX>
		.ENDIF
	.ENDIF

	.IF	al > 4

		emit	<CPU = 80>, nocr

		push	dx
		mov	dl, '0'
		add	dl, al
		call	EmitChar_RM
		pop	dx

		emit	<86>
	.ENDIF

                ret

_Emit = 0

Emit_CPU_NPX	endp

POPCONTEXT CPU

;
